package org.fmi.bdi.document;

import java.io.OutputStream;
import java.io.Writer;

import org.fmi.bdi.helpers.BDIException;
import org.fmi.bdi.helpers.BaseJAXBDoc;

/**
 * @author IvoJekov
 *
 *	Generic wrapper of XML document represented as set of JAXB objects
 */
public abstract class GenericBDIDocument extends BaseJAXBDoc
{
	// JAXB objects, which is the root of JAXB XML tree
	private Object fRootElement = null;
	
	/**
	 * see unMarshal(Object xml)
	 *
	 * @param the
	 *            xml is passed as an Object to the jaxb root object so that
	 *            the class model can be created.
	 */
	public GenericBDIDocument(Object xml, String contextPath) throws BDIException
	{
		setContextPath(contextPath);
		unMarshal(xml);
	}

	/**
	 * see unMarshal(Object xml)
	 *
	 * @param the
	 *            xml is passed as an String to the jaxb root object so that
	 *            the class model can be created.
	 */
	public GenericBDIDocument(String xml, String contextPath) throws BDIException
	{
		setContextPath(contextPath);
		unMarshal((Object) xml);
	}

	/**
	 * does not create underlying class model
	 *
	 */
	public GenericBDIDocument() throws BDIException
	{
		setContextPath(getContextPackage());
	}
	
	/**
	 *  Should return corresponding JAXB full package name
	 *
	 */ 
	protected abstract String getContextPackage();
	
	public void unMarshal(Object xml) throws BDIException
	{
		try
		{
			setRootXMLElement(genericUnMarshal(xml));
		}
		catch (Exception e)
		{
			throw new BDIException("unMarshal", e);
		}
	}
	
	protected Object getRootXMLElement()
	{
		return fRootElement;
	}
	
	protected void setRootXMLElement(Object rootElement)
	{
		fRootElement = rootElement;
	}
	
	
	/**
	 * use this method to write BDI Document to an output stream
	 *
	 * @param the
	 *            stream to which the xml is written
	 */
	public void marshal(OutputStream out) throws BDIException
	{
		marshal(out, getRootXMLElement());
	}

	/**
	 * use this method to write BDI Document to a writer. <br><b>Hint:
	 * </b> when you wish for the xml as a string use a StringWriter
	 *
	 * @param the
	 *            Writer to which the xml is written
	 */
	public void marshal(Writer out) throws BDIException
	{
		marshal(out, getRootXMLElement());
	}
}
