/*
 * Created on 2004-10-6
 *
 */
package org.fmi.bdi.document;

import java.io.File;

import org.fmi.bdi.db.DBHelper;
import org.fmi.bdi.db.domains.IValidationType;
import org.fmi.bdi.helpers.AppContext;
import org.fmi.bdi.helpers.BDIException;
import org.fmi.bdi.helpers.DocumentType;
import org.fmi.bdi.helpers.SourceSystem;
import org.fmi.bdi.parser.DTDValidationParser;
import org.fmi.bdi.parser.XSDValidationParser;
import org.w3c.dom.Document;

/**
 * @author IvoJekov
 *
 *	Wraps incoming and outgoing document together with database values,
 *	whic has been setup for that kind of document 
 */
public class GenericBDIDocumentWrapper
{
	// T_DOCUMENT.DOCUMENT_ID
	protected int fDocumentID;
	
	// MESSAGE_ID of incoming document
	protected int fIncomingMessageID;
	// MESSAGE_ID of outgoing document
	protected int fTransformedMessageID;
	
	// info about the source system, which has sent this document
	protected SourceSystem fSourceSystem;
	
	// validation type
	protected int fValidationTypeID;
	protected String fValidationTypeCode;
	protected String fValidationSource;
	
	// document type
	protected DocumentType fDocType;

	// transformation info
	protected int fTransformerID;
	protected String fTransformerJavaClass;	
	
	// incoming XML document
	protected Document fIncomingDocument;
	// the document in the internal format
	protected Document fTransformedDocument;
	
	public GenericBDIDocumentWrapper(int sourceSystemID,
										int docTypeId,
										File uploadedXML) throws BDIException
	{
		DBHelper.populateDocumentInfo(sourceSystemID,
									 docTypeId,
									 this);
		
		// validate XML document
		parseIncomingDocument(uploadedXML);
	}
	
	/**
	 * Parses and validates incoming document, if needed.
	 * Validation type of the document is considered.
	 * 
	 * @param uploadedXML	incomig file
	 * @throws BDIException
	 */
	protected void parseIncomingDocument(File uploadedXML) throws BDIException
	{
		AppContext app = AppContext.getInstance();
		String valSource = app.getRootValidationDir();
		if (! valSource.endsWith("/"))
			valSource += "/";
		if (fValidationSource != null)
			valSource += fValidationSource;
		
		switch (fValidationTypeID)
		{
			case IValidationType.DTD:
				DTDValidationParser dtdParser = new DTDValidationParser(valSource);
				fIncomingDocument = dtdParser.parse(uploadedXML);
				break;
			case IValidationType.XSD:
				XSDValidationParser xsdParser = new XSDValidationParser("file:/" + valSource);
				fIncomingDocument = xsdParser.parse(uploadedXML);
				break;
			default:
				throw new BDIException("Validation type: '" 
										+ fValidationTypeCode +"' not implemented.");
		}
	}
	
	/**
	 * @return Returns the fDocument.
	 */
	public Document getIncomingDocument()
	{
		return fIncomingDocument;
	}

	/**
	 * @param document The fDocument to set.
	 */
	public Document getTransformedDocument()
	{
		return fTransformedDocument;
	}

	/**
	 */
	public void setTransformedDocument(Document doc)
	{
		fTransformedDocument = doc;
	}
	
	/**
	 * @return Returns the fDocumentID.
	 */
	public int getDocumentID()
	{
		return fDocumentID;
	}

	/**
	 * @param documentID The fDocumentID to set.
	 */
	public void setDocumentID(int documentID)
	{
		fDocumentID = documentID;
	}

	/**
	 * @return Returns the fMessageID.
	 */
	public int getIncomingMessageID()
	{
		return fIncomingMessageID;
	}

	/**
	 * @param messageID The fMessageID to set.
	 */
	public void setIncomingMessageID(int messageID)
	{
		fIncomingMessageID = messageID;
	}

	/**
	 * @return Returns the fMessageID.
	 */
	public int getTransformedMessageID()
	{
		return fTransformedMessageID;
	}

	/**
	 * @param messageID The fMessageID to set.
	 */
	public void setTransformedMessageID(int messageID)
	{
		fTransformedMessageID = messageID;
	}
	
	/**
	 * @return Returns the fTransformerID.
	 */
	public int getTransformerID()
	{
		return fTransformerID;
	}

	/**
	 * @param transformerID The fTransformerID to set.
	 */
	public void setTransformerID(int transformerID)
	{
		fTransformerID = transformerID;
	}

	/**
	 * @return Returns the fTransformerJavaClass.
	 */
	public String getTransformerJavaClass()
	{
		return fTransformerJavaClass;
	}

	/**
	 * @param transformerJavaClass The fTransformerJavaClass to set.
	 */
	public void setTransformerJavaClass(String transformerJavaClass)
	{
		fTransformerJavaClass = transformerJavaClass;
	}

	/**
	 * @return Returns the fValidationSource.
	 */
	public String getValidationSource()
	{
		return fValidationSource;
	}

	/**
	 * @param validationSource The fValidationSource to set.
	 */
	public void setValidationSource(String validationSource)
	{
		fValidationSource = validationSource;
	}

	/**
	 * @return Returns the fValidationTypeCode.
	 */
	public String getValidationTypeCode()
	{
		return fValidationTypeCode;
	}

	/**
	 * @param validationTypeCode The fValidationTypeCode to set.
	 */
	public void setValidationTypeCode(String validationTypeCode)
	{
		fValidationTypeCode = validationTypeCode;
	}

	/**
	 * @return Returns the fValidationTypeID.
	 */
	public int getValidationTypeID()
	{
		return fValidationTypeID;
	}

	/**
	 * @param validationTypeID The fValidationTypeID to set.
	 */
	public void setValidationTypeID(int validationTypeID)
	{
		fValidationTypeID = validationTypeID;
	}

	
	/**
	 * @param docType The fDocType to set.
	 */
	public void setDocType(DocumentType docType)
	{
		fDocType = docType;
	}

	/**
	 * @param sourceSystem The fSourceSystem to set.
	 */
	public void setSourceSystem(SourceSystem sourceSystem)
	{
		fSourceSystem = sourceSystem;
	}

	public String getSourceSystemNameVersion()
	{
		return fSourceSystem.getNameVersion();
	}
}
