/*
 * Created on 2004-10-5
 *
 */
package org.fmi.bdi.document.order;

import java.io.Reader;
import java.io.StringReader;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.UnmarshalException;
import javax.xml.bind.Unmarshaller;

import org.fmi.bdi.commonstandard.orders.jaxb.Address;
import org.fmi.bdi.commonstandard.orders.jaxb.BDIDocument;
import org.fmi.bdi.commonstandard.orders.jaxb.BusinessPartner;
import org.fmi.bdi.commonstandard.orders.jaxb.CardInfo;
import org.fmi.bdi.commonstandard.orders.jaxb.ContactInfo;
import org.fmi.bdi.commonstandard.orders.jaxb.CurrencyType;
import org.fmi.bdi.commonstandard.orders.jaxb.CustomerReference;
import org.fmi.bdi.commonstandard.orders.jaxb.DateInfo;
import org.fmi.bdi.commonstandard.orders.jaxb.DateTypeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.Delivery;
import org.fmi.bdi.commonstandard.orders.jaxb.Discount;
import org.fmi.bdi.commonstandard.orders.jaxb.DiscountTypeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.Document;
import org.fmi.bdi.commonstandard.orders.jaxb.DocumentReference;
import org.fmi.bdi.commonstandard.orders.jaxb.IncomingMessageInfo;
import org.fmi.bdi.commonstandard.orders.jaxb.InstrTypeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.Instructions;
import org.fmi.bdi.commonstandard.orders.jaxb.Item;
import org.fmi.bdi.commonstandard.orders.jaxb.ObjectFactory;
import org.fmi.bdi.commonstandard.orders.jaxb.OrderHeader;
import org.fmi.bdi.commonstandard.orders.jaxb.OrderItem;
import org.fmi.bdi.commonstandard.orders.jaxb.OrderSummary;
import org.fmi.bdi.commonstandard.orders.jaxb.PartnerSystem;
import org.fmi.bdi.commonstandard.orders.jaxb.Payment;
import org.fmi.bdi.commonstandard.orders.jaxb.Price;
import org.fmi.bdi.commonstandard.orders.jaxb.PriceTypeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.ProductCodeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.ProductInformation;
import org.fmi.bdi.commonstandard.orders.jaxb.PurchaseOrderType;
import org.fmi.bdi.commonstandard.orders.jaxb.Quantity;
import org.fmi.bdi.commonstandard.orders.jaxb.Tax;
import org.fmi.bdi.commonstandard.orders.jaxb.Transport;
import org.fmi.bdi.commonstandard.orders.jaxb.TypeIDConstants;
import org.fmi.bdi.document.GenericBDIDocument;
import org.fmi.bdi.helpers.AppContext;
import org.fmi.bdi.helpers.BDIException;
import org.xml.sax.InputSource;
import org.xml.sax.SAXParseException;

/**
 * @author IvoJekov
 *
 */
public class OrderBDIDocument extends GenericBDIDocument
{
	/**
	 * The ObjectFactory associated with this document <br>The Factory is used
	 * to create Objects within the document
	 */
	private static ObjectFactory fGSFactory = new ObjectFactory();

	/**
	 * see getContextPackage() in GenericBDIDocument()
	 */
	protected String getContextPackage()
	{
		// return the property, which has been obtained from app configuration
		return AppContext.getInstance().getJaxbOrdersPackage();
	}
	
	/**
	 * see unMarshal(Object xml)
	 *
	 * @param the
	 *            xml is passed as an Object to the jaxb root object so that
	 *            the class model can be created.
	 */
	public OrderBDIDocument(String xml, String contextPath) throws BDIException
	{
		super(xml, contextPath);
	}


	/**
	 * Initialize the document and creates some of the mandatory elements
	 * @throws BDIException
	 */
	public OrderBDIDocument() throws BDIException
	{
		super();
		try
		{
			setJaxbContext(JAXBContext.newInstance(getContextPath()));
			setRoot(getGSFactory().createBDIDocument());

			// mandatory parameters should be set here
			// header is mandatory
			getRoot().setHeader(getGSFactory().createHeaderType());
			IncomingMessageInfo msgInfo = getGSFactory().createIncomingMessageInfo();
			getRoot().getHeader().setIncomingMessageInfo(msgInfo);

			// header must have Document
			Document doc = getGSFactory().createDocument();
			doc.setDocType("PurchaseOrder");
			doc.setProprietaryDocID("");
			getRoot().getHeader().setDocument(doc);

			// header must have delivery info
			// and delivery must have to and from partner info 
			PartnerSystem to = getGSFactory().createPartnerSystem();
			PartnerSystem from = getGSFactory().createPartnerSystem();
			
			Delivery delivery = getGSFactory().createDelivery();
			delivery.setFrom(from);
			delivery.setTo(to);
			getRoot().getHeader().setDelivery(delivery);
			
			// body
			getRoot().setBody(getGSFactory().createBody());
			getRoot().getBody().setPurchaseOrder(
					getGSFactory().createPurchaseOrder());

			// purchase order must have currency
			CurrencyType primaryCurrency = getGSFactory().createCurrencyType();
			// should be populated from ancestors
			primaryCurrency.setPrimaryCurrency("");
			getPurchaseOrder().setCurrency(primaryCurrency);
		}
		catch (JAXBException e)
		{
			throw new BDIException(
					"Failed to initialize while creating the root: " + e.getMessage(),
					e);
		}
	}

	

	/**
	 *
	 */
	protected Object unMarshal(String xml) throws BDIException
	{
		try
		{
			setJaxbContext(JAXBContext.newInstance(getContextPath()));
			Unmarshaller unMarshaller = getJaxbContext().createUnmarshaller();
			unMarshaller.setValidating(true);
			
			Reader reader = new StringReader(xml);
			InputSource is = new InputSource();
			is.setCharacterStream(reader);
			return unMarshaller.unmarshal(is);
		}
		catch (UnmarshalException e)
		{
			String msg = e.toString();
			SAXParseException saxe = null;
			if (e.getLinkedException() != null
					&& e.getLinkedException() instanceof SAXParseException)
				saxe = (SAXParseException) e.getLinkedException();

			if (saxe != null)
				msg += "Line number "
					+ saxe.getLineNumber()
					+ " Column "
					+ saxe.getColumnNumber();

			throw new BDIException("unMarshal: " + msg, e);
		}
		catch (Exception e)
		{
			throw new BDIException(
					"unMarshal: Unexpected Error raised: " + e.getMessage(),
					e);
		}
	}
	
	/**
	 * The ObjectFactory is the factory with which to create jaxb objects for
	 * this document.
	 *
	 * @return the ObjectFactory associated with this document
	 */

	public ObjectFactory getGSFactory()
	{
		return fGSFactory;
	}

	/**
	 * Internal method.
	 *
	 * @param object
	 *            factory to use.
	 */

	protected void setGSFactory(ObjectFactory factory)
	{
		fGSFactory = factory;
	}
	
	/**
	 * This is a key into the root of the jaxb object model. From this object
	 * any child class can be accessed.
	 *
	 * @return the jaxb root object.
	 */
	public BDIDocument getRoot()
	{
		return (BDIDocument) getRootXMLElement();
	}

	/**
	 * Internal method: Sets the jaxb root object for this document.
	 *
	 * @param bizTalkRoot
	 *            The bizTalkRoot to set
	 */
	public void setRoot(BDIDocument root)
	{
		setRootXMLElement(root);
	}

	/**
	 * Marshalls JAXB representation as DOM Tree
	 * 
	 * @return resulting DOM document
	 * @throws BDIException
	 */
	public org.w3c.dom.Document getTransformedDocument() throws BDIException
	{
		return marshal(getRoot());
	}
	
	
	public PurchaseOrderType getPurchaseOrder()
	{
		return getRoot().getBody().getPurchaseOrder();
	}
	
	//////////////////////////////////////////////////////////////
	// helpers
	//////////////////////////////////////////////////////////////
	
	/**
	 * Set BDIDocument/Header/Delivery/To
	 */
	public void setToPartner(String gsID,
							String name,
							String division) throws JAXBException
	{
		PartnerSystem to = producePartnerSystem(gsID, name, division);
		getRoot().getHeader().getDelivery().setTo(to);
	}

	/**
	 * Set BDIDocument/Header/Delivery/From
	 * 
	 * @param gsID
	 * @param name
	 * @param division
	 * @throws JAXBException
	 */	
	public void setFromPartner(String gsID,
								String name,
								String division) throws JAXBException
	{
		PartnerSystem from = producePartnerSystem(gsID, name, division);
		getRoot().getHeader().getDelivery().setFrom(from);
	}
	
	
	private PartnerSystem producePartnerSystem(String gsID,
												String name,
												String division) throws JAXBException
	{
		PartnerSystem partner = getGSFactory().createPartnerSystem();
		partner.setGlobalSystemIdentifier(gsID);
		if (name != null)
			partner.setName(name);
		if (division != null)
			partner.setDivisionCode(division);
		
		return partner;
	}
	
	
	/**
	 * Set BDIDocument/Body/PurchaseOrder/CardInfo
	 * 
	 * @param cardType
	 * @param cardNum
	 * @param holderName
	 * @param expirationDate
	 * @throws JAXBException
	 */	
	public void setCardInfo(String cardType, 
							String cardNum,
							String holderName,
							String expirationDate) throws JAXBException
	{
		CardInfo cardInfo = getGSFactory().createCardInfo();
		if (cardType != null)
			cardInfo.setCardType(cardType);
		cardInfo.setCardNum(cardNum);
		if (expirationDate != null)
			cardInfo.setCardExpirationDate(expirationDate);
		if (holderName != null)
			cardInfo.setCardHolderName(holderName);
		
		getPurchaseOrder().setCardInfo(cardInfo);
	}

	/**
	 * Set BDIDocument/Body/PurchaseOrder/Tax
	 * 
	 * @param primaryCur
	 * @param secondaryCur
	 * @param exchangeRate
	 * @param taxAmount
	 * @param descr
	 * @throws JAXBException
	 * @throws BDIException
	 */
	public void setTax(String primaryCur,
					  String secondaryCur,
					  String exchangeRate,
					  String taxAmount,
					  String descr) throws JAXBException, BDIException
	{
		double dAmount = 0;
		try
		{
			dAmount = Double.parseDouble(taxAmount);
		}
		catch (NumberFormatException nfe)
		{
			// nothing to due
			throw new BDIException("Invalid tax supplied " 
					+ nfe.getMessage(), 
					nfe);
		}
		
		CurrencyType currency = produceCurrency(primaryCur, secondaryCur, exchangeRate);
		Tax tax = getGSFactory().createTax();
		if (taxAmount != null)
			tax.setTaxAmount(dAmount);
		if (descr != null)
			tax.setTaxDescription(descr);
		if (currency != null)
			tax.setTaxCurrency(currency);
		
		getPurchaseOrder().setTax(tax);
	}
	
	/**
	 * Set BDIDocument/Body/PurchaseOrder/Transport
	 * 
	 * @param item
	 * @param transpDate
	 * @param transpMode
	 * @throws JAXBException
	 */	
	public void setTransport(Item item, 
							String transpDate, 
							String transpMode) throws JAXBException
	{
		Transport transp = getGSFactory().createTransport();
		transp.setTransportDate(transpDate);
		transp.setTransportMode(transpMode);
		
		if (item != null)
			item.setTransport(transp);
		else
			getPurchaseOrder().setTransport(transp);
	}	

	/**
	 * Add an BDIDocument/Body/OrderHeader
	 * 
	 * @param orderType
	 * @param businessID
	 * @param businessDesc
	 * @throws JAXBException
	 */
	public void addOrderHeader(String orderType,	
								String businessID,
								String businessDesc) throws JAXBException
	{
		// purchase order must have OrderHeader
		List ohInfos = getPurchaseOrder().getOrderHeader();
		
		OrderHeader oh = getGSFactory().createOrderHeader();
		oh.setOrderType(orderType);
		if (businessID != null)
			oh.setBusinessID(businessID);
		if (businessDesc != null)
			oh.setBusinessDesc(businessDesc);
		
		ohInfos.add(oh);
	}

	/**
	 * Set BDIDocument/Body/PurchaseOrder/OrderSummary
	 * 
	 * @param priceTotal
	 * @param secondaryPriceTotal
	 * @param lineCount
	 * @throws JAXBException
	 */
	public void setOrderSummary(float priceTotal,	
								String secondaryPriceTotal,
								int lineCount) throws JAXBException
	{
		OrderSummary os = getGSFactory().createOrderSummary();
		os.setPrimaryCurrencyTotal(priceTotal);
		if (secondaryPriceTotal != null)
			os.setSecondaryCurrencyTotal(Float.parseFloat(secondaryPriceTotal));
		os.setTotalLinesCount(lineCount);

		getPurchaseOrder().setOrderSummary(os);
	}
	
	/**
	 * Set BDIDocument/Body/PurchaseOrder/OrderItem/Item/Quantity
	 * 
	 * @param countryCode
	 * @param state
	 * @param city
	 * @param addressLines
	 * @param zipCode
	 * @return
	 * @throws JAXBException
	 */
	public void setItemQuantity(Item item,
								int quantity,
								String uom) 
				throws JAXBException, BDIException
	{
		Quantity itemQuan = getGSFactory().createQuantity();
		itemQuan.setTotalQuantity(quantity);
		if (uom != null)
			itemQuan.setUnitOfMeasure(uom);
		
		item.setQuantity(itemQuan);
	}
	
	
	public void addItemPrice(Item item,
							float price,
							String priceDescr,
							PriceTypeConstants type) 
					throws JAXBException
	{
		Price itemPrice = getGSFactory().createPrice();
		itemPrice.setPriceAmount(price);
		itemPrice.setPriceType(type);
		if (priceDescr != null)
			itemPrice.setPriceDescription(priceDescr);
		
		item.getPrice().add(itemPrice);
	}

	public void addItemProductInfo(Item item,
									ProductCodeConstants prodCode,			
									String prodNum,
									String prodCommodityCode,
									String prodName,
									String prodDescr) throws JAXBException
	{
		List prodInfos = item.getProductInformation();
		
		ProductInformation prodInfo = getGSFactory().createProductInformation();
		prodInfo.setProductNumber(prodNum);
		if (prodCommodityCode != null)
			prodInfo.setProductCommodityCode(prodCommodityCode);
		prodInfo.setProductCode(prodCode);
		if (prodName != null)
			prodInfo.setProductName(prodName);
		if (prodDescr != null)
			prodInfo.setProductDescription(prodDescr);
		
		item.getProductInformation().add(prodInfo);
	}
	
	public void addDateInfoC(Item item,
							DateTypeConstants type,
							String dateTime,
							String dateDescr)
				throws JAXBException
	{
		DateInfo date = getGSFactory().createDateInfo();
		date.setDateType(type);
		date.setDateTimeUTC(dateTime);
		if (dateDescr != null)
			date.setDateDesc(dateDescr);
		
		if (item != null)
			item.getDateInfo().add(date);
		else
			getPurchaseOrder().getDateInfo().add(date);
	}
	
	public void setCurrency(Item item,
							String primary,
							String secondary,
							String exchangeRate)
					throws JAXBException, BDIException
	{
		CurrencyType cur = produceCurrency(primary, secondary, exchangeRate);

		if (item != null)
			item.setCurrency(cur);
		else
			getPurchaseOrder().setCurrency(cur);
	}
	
	public void addPayment(String type,	
			String code,
			String term,
			String termOther) throws JAXBException
	{
		List payments = getPurchaseOrder().getPayment();
		
		Payment payment = getGSFactory().createPayment();
		payment.setPaymentType(type);
		payment.setPaymenTermCode(code);
		if (term != null)
			payment.setPaymenTerm(term);
		if (termOther != null)
			payment.setPaymenTermOther(termOther);
		
		payments.add(payment);
	}
	
	public void addDiscount(DiscountTypeConstants type,
							String id,
							String code,
							String amount,
							String percentage,
							String dueDate,
							String duration) throws JAXBException, BDIException
	{
		List discounts = getPurchaseOrder().getDiscount();
		
		int dur = -1;
		float perc = 0.0f;
		try
		{
			perc = Float.parseFloat(percentage);
			dur = Integer.parseInt(duration);
		}
		catch (NumberFormatException nfe)
		{
			// nothing to due
			// discount will be ignored
			throw new BDIException("Invalid discount supplied: " + nfe.getMessage(), nfe);
		}
		
		Discount disc = getGSFactory().createDiscount();
		disc.setDiscountType(type);
		if (amount != null)
			disc.setDiscountAmount(Float.parseFloat(amount));
		if (percentage != null)
			disc.setDiscountPercent(perc);
		if (code != null)
			disc.setDiscountCode(code);
		if (id != null)
			disc.setDiscountID(id);
		if (duration != null)
			disc.setDiscountDaysDuration(dur);
		if (dueDate != null)
			disc.setDiscountDueDate(dueDate);
		
		discounts.add(disc);
	}
	
	public void addInstruction(Item item,
								InstrTypeConstants type,
								String internalCode,
								String code,
								String value,
								String codeDesc) throws JAXBException
	{
		List instructions = getPurchaseOrder().getInstructions();
		if (item != null)
		{
			// instructions are added at item level
			instructions = item.getInstructions();
		}
			
		Instructions instr = getGSFactory().createInstructions();
		instr.setInstrType(type);
		instr.setInternalCode(internalCode);
		instr.setInstrCode(code);
		instr.setInstrValue(value);
		
		if (codeDesc != null)
			instr.setInstrCodeDesc(codeDesc);
		
		instructions.add(instr);
			
	}
	
	public void addDocumentRef(Item item,
								String docId,
								String docType,
								String docCode,
								String docDesc) throws JAXBException
	{
		List docRefs = getPurchaseOrder().getDocumentReference();
		if (item != null)
		{
			// add at item level
			docRefs = item.getDocumentReference();
		}
		
		DocumentReference ref = getGSFactory().createDocumentReference();
		if (docType != null)
			ref.setDocType(docType);
		if (docCode != null)
			ref.setDocumentCode(docCode);
		if (docDesc != null)
			ref.setDocumentDesc(docDesc);
		ref.setDocumentID(docId);
				
		docRefs.add(ref);
	}	

	
	public Item produceOrderItem(int lineNumber) throws JAXBException
	{
		Item item = getGSFactory().createItem();
		item.setLineNumber(lineNumber);
		
		OrderItem newOI = getGSFactory().createOrderItem();
		newOI.setItem(item);
		getPurchaseOrder().getOrderItem().add(newOI);
		
		return item;
	}
	
	public ContactInfo produceContactInfo(String name, 
										String email, 
										String tel,
										String fax) throws JAXBException
	{
		boolean toProduceCI = false;
		
		ContactInfo ci = getGSFactory().createContactInfo();
		if (name != null)
		{
			toProduceCI = true;
			ci.setContactName(name);
		}
		
		if (email != null)
			ci.setContactEmail(email);
		if (tel != null)
			ci.setContactPhone(tel);
		if (fax != null)
			ci.setContactFax(fax);
		
		if (toProduceCI)
			return ci;
		
		return null;
	}
	
	/**
	 * could return null, if all of the params are empty
	 */ 
	public Address produceAddress(String countryCode,
									String state,
									String city,
									List addressLines,
									String zipCode) throws JAXBException
	{
		boolean toProduceAddress = false;
		
		Address add = getGSFactory().createAddress();
		if (countryCode != null)
		{
			toProduceAddress = true;
			add.setISOCountryCode(countryCode);
		}
		
		if (city != null)
		{
			toProduceAddress = true;
			add.setCity(city);
		}
		
		if (state != null)
		{
			toProduceAddress = true;
			add.setState(state);
		}
		
		if (zipCode != null)
		{
			toProduceAddress = true;
			add.setZIPCode(zipCode);
		}
		
		if (addressLines != null && !addressLines.isEmpty())
		{
			toProduceAddress = true;
			add.getStreet().addAll(addressLines);
		}
		
		if (toProduceAddress) 
			return add;
	
		return null;
	}
	
	/**
	 * @param orderItem
	 * @param type
	 * @param partnerName
	 * @param partnerCode
	 * @param partnerID
	 * @param contactInfos
	 * @param address
	 * @throws JAXBException
	 */
	public void addCustomerRef(Item orderItem,
			TypeIDConstants type,
			String partnerName,
			String partnerCode,
			String partnerID,
			List contactInfos,
			Address address) throws JAXBException
	{
		CustomerReference ref = produceCustomerRef(type,
				partnerName,
				partnerCode,
				partnerID,
				contactInfos,
				address);
		
		if (ref != null)
		{
			if (orderItem != null)
				orderItem.getCustomerReference().add(ref);
			else
				getPurchaseOrder(). getCustomerReference().add(ref);
		}
	}	
	
	private CustomerReference produceCustomerRef(TypeIDConstants type,
			String partnerName,
			String partnerCode,
			String partnerID,
			List contactInfos,
			Address address) throws JAXBException
	{
		BusinessPartner bp = getGSFactory().createBusinessPartner();
		bp.setTypeID(type);
		if (partnerCode != null)
			bp.setPartnerCode(partnerCode);
		if (partnerID != null)
			bp.setPartnerID(partnerID);
		if (partnerName != null)
			bp.setPartnerName(partnerName);

		CustomerReference ref = getGSFactory().createCustomerReference();
		ref.setBusinessPartner(bp);
		if (contactInfos != null && !contactInfos.isEmpty())
			ref.getContactInfo().addAll(contactInfos);
		if (address != null)
			ref.setAddress(address);
		
		return ref;
	}
	
	public CurrencyType produceCurrency(String primary,
										String secondary,
										String exchangeRate)
				throws JAXBException, BDIException
	{
		float fExchRate = -1f;
		try
		{
			if (exchangeRate != null)
				fExchRate = Float.parseFloat(exchangeRate);
		}
		catch (NumberFormatException nfe)
		{
			// nothing to due
			throw new BDIException("Invalid exchenge rate supplied " 
					+ nfe.getMessage(), 
					nfe);
		}
		
		CurrencyType cur = getGSFactory().createCurrencyType();
		cur.setPrimaryCurrency(primary);
		if (secondary != null)
			cur.setSecondaryCurrency(secondary);
		if (fExchRate != -1)
			cur.setExchangeRate(fExchRate);
		
		return cur;
	}
	
}
