/*
 * Created on 2004-8-13
 *
 */
package org.fmi.bdi.parser;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import org.apache.xerces.parsers.DOMParser;
import org.fmi.bdi.helpers.BDIException;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * @author IvoJekov
 *
 * Parses and validates an XML document. 
 * Successors takes care about the validation source's type
 */
public class ValidationParser
{
	public static final String FEATURE_NAMESPACE 					= 	"http://xml.org/sax/features/namespaces";
	public static final String FEATURE_VALIDATION					= 	"http://xml.org/sax/features/validation";

	// specific features for XSD validation
	public static final String FEATURE_VALIDATION_SCHEMA 			= 	"http://apache.org/xml/features/validation/schema";
	public static final String FEATURE_VALIDATION_SCHEMA_FULL_CHECK = 	"http://apache.org/xml/features/validation/schema-full-checking";
	
	public static final String PROPERTY_VALIDATION_SCHEMA_LOCATION 	= 	"http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation";
	
	protected String fValidationSourceURI;
	// underlying parser
	protected DOMParser fParser = null;
	// error handler
	protected ValidationErrorHandler fErrorHandler = null;
	
	public ValidationParser(String validationSourceURI) throws BDIException
	{
		fValidationSourceURI = validationSourceURI;
		fParser = new DOMParser();
		
		try
		{
			fParser.setFeature(FEATURE_NAMESPACE, true);
			fParser.setFeature(FEATURE_VALIDATION, true);
		} 
		catch (Exception saxe)
		{
			throw new BDIException("Error initializing parser:", saxe);
		}

		fErrorHandler = new ValidationErrorHandler();
		fParser.setErrorHandler(fErrorHandler);
	}
	
	/**
	 * Parse and validate incoming XML
	 * 
	 * @param xml incoming XML
	 * @return parsed and validated  document
	 * @throws BDIException if validation fails
	 */
	public Document parse(File xml) throws BDIException
	{
		try
		{
			fParser.parse(new InputSource(new FileInputStream(xml)));
			if (fErrorHandler.hasErrors())
			{
				String valRes = fErrorHandler.getValidationResult();
				throw new BDIException("Error parsing file: " + valRes);
			}
		}
		catch (IOException io)
		{
			throw new BDIException("File not found:", io);
		}
		catch (SAXException saxe)
		{
			throw new BDIException("Error parsing file", saxe);
		}
		
		return fParser.getDocument();
	}
	
}
