/*
 * Created on 2004-10-6
 *
 */
package org.fmi.bdi.transformer.order;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBException;
import javax.xml.transform.TransformerException;

import org.apache.xpath.XPathAPI;
import org.fmi.bdi.commonstandard.orders.jaxb.Address;
import org.fmi.bdi.commonstandard.orders.jaxb.ContactInfo;
import org.fmi.bdi.commonstandard.orders.jaxb.DateTypeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.InstrTypeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.Item;
import org.fmi.bdi.commonstandard.orders.jaxb.PriceTypeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.ProductCodeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.TypeIDConstants;
import org.fmi.bdi.helpers.BDIException;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author IvoJekov
 *
 *	Transforms incoming cXML document into the internal XML format
 */
public class CXMLTransformer extends GenericXMLOrderTransformer
{

	private float fTotalPrice = 0;
	private float fAlternateTotalPrice = 0;
	
	public CXMLTransformer() throws BDIException
	{
		super();
	}
	
	/* (non-Javadoc)
	 * @see org.fmi.bdi.transformer.order.GenericOrderTransformer#performTransformation(org.fmi.bdi.document.GenericBDIDocumentWrapper)
	 */
	protected void performTransformation() throws BDIException
	{
		try
		{
			populateToFromInfo();
			populateHeaderInfo();
			populateOrderCurrencyInfo();
			populateDocumentRef();
			populateCustRef();
			populateCardInfo();
			
			populateOrderItemsAndSummary();
			
			populateTaxInfo();
		}
		catch (BDIException e)
		{
			throw e;
		}
		catch (Exception e)
		{
			throw new BDIException("Exception while transforming to internal format: " + e.getMessage(),
					e);
		}
	}
	
	
	private void populateToFromInfo() throws TransformerException, JAXBException
	{
		final String FROM_ORG_PATH = "/cXML/Header/From/Credential";
		final String TO_ORG_PATH = "/cXML/Header/To/Credential";		
		final String GLOBAL_SYSTEM_PATH = "Identity";
		final String NAME_PATH = "@domain";
		final String DIVISION_PATH = "@type";
		
		// To parnter System
		Node to = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), TO_ORG_PATH);
		String toGSID = getNodeValue(to, GLOBAL_SYSTEM_PATH);
		String toName = getNodeValue(to, NAME_PATH);
		String toDivision = getNodeValue(to, DIVISION_PATH);
		fJAXBWrapper.setToPartner(toGSID, toName, toDivision);
		
		// From Partner System
		Node from = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), FROM_ORG_PATH);
		String fromGSID = getNodeValue(from, GLOBAL_SYSTEM_PATH);
		String fromName = getNodeValue(from, NAME_PATH);
		String fromDivision = getNodeValue(from, DIVISION_PATH);		
		fJAXBWrapper.setFromPartner(fromGSID, fromName, fromDivision);
	}
	
	private void populateHeaderInfo() throws TransformerException, JAXBException
	{
		final String DOC_PATH = "/cXML/Request/OrderRequest/OrderRequestHeader";
		final String DOC_ID_PATH = "@orderID";
		final String SEND_DATE_PATH = "@orderDate";
		final String ORDER_TYPE_PATH = "@orderType";
		final String TYPE_PATH = "@type";
		
		// orderID and orderDate are required
		Node docNode = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), DOC_PATH);
		String orderId = getNodeValue(docNode, DOC_ID_PATH);
		String orderDate = getNodeValue(docNode, SEND_DATE_PATH); 
		String type = getNodeValue(docNode, TYPE_PATH);
		if (type == null)
			type = getNodeValue(docNode, ORDER_TYPE_PATH); 
		
		fJAXBWrapper.getRoot().getHeader().getDocument().setProprietaryDocID(orderId);
		fJAXBWrapper.getRoot().getHeader().getIncomingMessageInfo().setSentDate(orderDate);
		fJAXBWrapper.addOrderHeader(type, null, null);
	}
	
	private void populateOrderCurrencyInfo() throws TransformerException, 
													JAXBException,
													BDIException
	{
		final String CURRENCY_PATH = "/cXML/Request/OrderRequest/OrderRequestHeader/Total/Money";
		final String PRIMARY_SUBPATH = "@currency";
		final String SECONDARY_SUBPATH = "@alternateCurrency";
		final String ALTERNATE_AMOUNT_SUBPATH = "@alternateAmount";
		
		Node currencyNode = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), CURRENCY_PATH);
		// could not be null
		String currency = getNodeValue(currencyNode, PRIMARY_SUBPATH);
		// could be null
		String alternateCurrency = getNodeValue(currencyNode, SECONDARY_SUBPATH);
		fJAXBWrapper.setCurrency(null, currency, alternateCurrency, null);
		
		try
		{
			fTotalPrice = Float.parseFloat(currencyNode.getFirstChild().getNodeValue());
		}
		catch (NumberFormatException nfe)
		{
			throw new BDIException("Invalid total price supplied: " + nfe.getMessage(), nfe);
		}
		
		String altAmount = getNodeValue(currencyNode, ALTERNATE_AMOUNT_SUBPATH);
		try
		{
			if (altAmount != null)
				fAlternateTotalPrice = Float.parseFloat(altAmount);
		}
		catch (NumberFormatException nfe)
		{
			throw new BDIException("Invalid alternate total price supplied: " + nfe.getMessage(), nfe);
		}
		
	}
	
	private void populateDocumentRef() throws TransformerException, JAXBException
	{
		final String DOC_REF_PATH = "/cXML/Request/OrderRequest/OrderRequestHeader/DocumentReference";
		final String DOC_ID_SUBPATH = "@payloadID";

		
		// could be null
		Node docRef = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), DOC_REF_PATH);
		if (docRef != null)
		{	
			String id = getNodeValue(docRef, DOC_ID_SUBPATH);
			
			fJAXBWrapper.addDocumentRef(null, id, null, null, null);
		}
	}
	
	private void populateCardInfo() throws TransformerException, JAXBException
	{
		final String CREDIT_CARD_PATH = "/cXML/Request/OrderRequest/OrderRequestHeader/Payment/PCard";
		final String CARD_NUM_SUBPATH = "@number";
		final String EXPIRATION_DATE_SUBPATH = "@expiration";	
		final String HOLDER_NAME_SUBPATH = "@name";	
		
		// could be null
		Node creditCard = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), 
													CREDIT_CARD_PATH);
		if (creditCard != null)
		{
			String cardNum = getNodeValue(creditCard, CARD_NUM_SUBPATH);
			String expDate = getNodeValue(creditCard, EXPIRATION_DATE_SUBPATH);
			String holderName = getNodeValue(creditCard, HOLDER_NAME_SUBPATH);
			
			fJAXBWrapper.setCardInfo(null, cardNum, holderName, expDate);			
		}
	}
	
	private void populateTaxInfo() throws TransformerException, 
										JAXBException,
										BDIException
	{
		final String TAX_PATH = "/cXML/Request/OrderRequest/OrderRequestHeader/Tax";
		final String DESCRIPTION_SUBPATH = "Description";
		final String CURRENCY_SUBPATH = "Money";
		final String PRIMARY_SUBPATH = "@currency";
		final String SECONDARY_SUBPATH = "@alternateCurrency";

		// could be null
		Node tax = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), 
				TAX_PATH);
		if (tax != null)
		{
			Node moneyNode = XPathAPI.selectSingleNode(tax, CURRENCY_SUBPATH);
			// can not be null
			String amount = moneyNode.getFirstChild().getNodeValue();
			// could not be null
			String currency = getNodeValue(moneyNode, PRIMARY_SUBPATH);
			// could be null
			String alternateCurrency = getNodeValue(moneyNode, SECONDARY_SUBPATH);
			// can not be null
			String description = getNodeValue(tax, DESCRIPTION_SUBPATH);
			
			fJAXBWrapper.setTax(currency, alternateCurrency, null, amount, description);
		}
	}
	
	private void populateCustRef() throws TransformerException, JAXBException
	{
		final String BILL_TO_PATH = "/cXML/Request/OrderRequest/OrderRequestHeader/BillTo/Address";
		final String SHIP_TO_PATH = "/cXML/Request/OrderRequest/OrderRequestHeader/ShipTo/Address";
		
		// 	bill to, can not be null
		Node billNode = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), 
				BILL_TO_PATH);
		processCustRef(billNode, TypeIDConstants.BIL, null);
		
		// ship to, could be null
		Node shipToNode = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), 
				SHIP_TO_PATH);
		processCustRef(shipToNode, TypeIDConstants.SHP, null);
	}
	
	private void populateOrderItemsAndSummary() throws TransformerException, 
														JAXBException,
														BDIException
	{
		final String ORDER_ITEMS_PATH = "/cXML/Request/OrderRequest/ItemOut";
		
		// CustRef
		final String SHIP_TO_SUBPATH = "ShipTo/Address";

		// Date Info, request deliveryDate
		final String DELIVERY_DATE_PATH = "@requestedDeliveryDate";
		
		// Quantity
		final String QUANTITY_SUBPATH = "@quantity";
		final String UOM_SUBPATH = "ItemDetail/UnitOfMeasure";

		// Line Number
		final String ITEM_NUMBER_SUBPATH = "@lineNumber";
		
		// Price
		final String PRICE_AMOUNT_SUBPATH = "ItemDetail/UnitPrice/Money";

		// Currency
		final String PRIMARY_CURRENCY_PATH = "@currency";
		final String SECONDARY_CURRENCY_PATH = "@alternateCurrency";
		
		// ProductInfo SUP
		final String PRODUCT_SUP_NUMBER_PATH = "ItemID/SupplierPartID";

		// ProductInfo Manifacture
		final String PRODUCT_MAN_NUMBER_PATH = "ItemDetail/ManufacturerPartID";
		final String PRODUCT_MAN_NAME_PATH = "ItemDetail/ManufacturerName";
		
		float totalPrice = 0;
		int orderItemsCnt = 0;
		
		NodeList orderItems = XPathAPI.selectNodeList(fBDIDocWrapper.getIncomingDocument(),
													ORDER_ITEMS_PATH);
		orderItemsCnt = orderItems.getLength();
		for (int i = 0; i < orderItemsCnt; i++)
		{
			Node curItem = orderItems.item(i);
			Item newOI = fJAXBWrapper.produceOrderItem(i + 1);
			
			// CustRef SHP
			Node shipToNode = XPathAPI.selectSingleNode(curItem, SHIP_TO_SUBPATH);
			processCustRef(shipToNode, TypeIDConstants.SHP, newOI);
			
			// DateInfo 
			// could be null
			String deliveryDate = getNodeValue(curItem, DELIVERY_DATE_PATH);
			if (deliveryDate != null)
			{
				fJAXBWrapper.addDateInfoC(newOI, DateTypeConstants.RDD, deliveryDate, null);
			}
			
			// ProductInfos
			// Supplier, can not be null
			String supplierProdNum = getNodeValue(curItem, PRODUCT_SUP_NUMBER_PATH);
			fJAXBWrapper.addItemProductInfo(newOI, 
					ProductCodeConstants.SPN, 
					supplierProdNum, 
					null, 
					null, 
					null);

			// Manifacture = VPN
			String manProdNum = getNodeValue(curItem, PRODUCT_MAN_NUMBER_PATH);
			String manName = getNodeValue(curItem, PRODUCT_MAN_NAME_PATH);
			fJAXBWrapper.addItemProductInfo(newOI, 
					ProductCodeConstants.VPN, 
					manProdNum, 
					null, 
					manName, 
					null);
			
			
			// Price
			String price = getNodeValue(curItem, PRICE_AMOUNT_SUBPATH);
			float curPrice = 0;
			try
			{
				curPrice = Float.parseFloat(price);
			}
			catch (NumberFormatException nfe)
			{
				// nothing to due
				throw new BDIException("Invalid price supplied " 
						+ nfe.getMessage() + 
						" at item: " + newOI.getLineNumber(), 
						nfe);
			}
			fJAXBWrapper.addItemPrice(newOI, curPrice, null, PriceTypeConstants.BDP);
			
			// Quantity
			String quantity = getNodeValue(curItem, QUANTITY_SUBPATH);
			int iQuantity = 0;
			try
			{
				iQuantity = Integer.parseInt(quantity);
			}
			catch (NumberFormatException nfe)
			{
				// nothing to due
				throw new BDIException("Invalid quantity supplied " 
						+ nfe.getMessage() + 
						" at item: " + newOI.getLineNumber(), 
						nfe);
			}
			String uom = getNodeValue(curItem, UOM_SUBPATH);
			fJAXBWrapper.setItemQuantity(newOI, iQuantity, uom);

			// calculate total order price
			totalPrice += iQuantity*curPrice;
			
			// Currency
			Node moneyNode = XPathAPI.selectSingleNode(curItem, PRICE_AMOUNT_SUBPATH);
			String currency = getNodeValue(moneyNode, PRIMARY_CURRENCY_PATH);
			String seconfaryCurrency = getNodeValue(moneyNode, SECONDARY_CURRENCY_PATH);
			if (currency != null)
				fJAXBWrapper.setCurrency(newOI, currency, seconfaryCurrency, null);
			
			// Item Number
			newOI.setItemNumber(getNodeValue(curItem, ITEM_NUMBER_SUBPATH));
		}
		
		// generate warning, if totalPrice != fTotal
		if (totalPrice != fTotalPrice)
		{
			String instrCode = "PriceDoNotMatch";
			String instrValue = "Summary price: " + fTotalPrice
								+ " do not match to the sum of item prices: " + totalPrice;
			fJAXBWrapper.addInstruction(null, 
										InstrTypeConstants.WARN, 
										"PRICE", 
										instrCode,
										instrValue,
										null);
		}
		
		String sAltTotal = null;
		if (fAlternateTotalPrice > 0)
		{
			sAltTotal = fAlternateTotalPrice + "";
		}
		
		fJAXBWrapper.setOrderSummary(totalPrice, sAltTotal, orderItemsCnt);
	}
	
	
	
	
	//////////////////////////////////////////////////////////////////
	//
	// common helpers here
	//
	/////////////////////////////////////////////////////////////////
	private void processCustRef(Node addressNode, 
											TypeIDConstants custRefType,
											Item item) 
				throws TransformerException, JAXBException
	{
		// business description
		final String BUSINESS_ID_PATH = "Name";
		
		// contact info
		final String CONTACT_NAME_PATH = "PostalAddress/DeliverTo";

		// address
		final String CITY_PATH = "PostalAddress/City";
		final String COUNTRY_PATH = "PostalAddress/Country";
		final String COUNTRY_CODE_PATH = "@isoCountryCode";
		final String STATE_PATH = "PostalAddress/State";
		final String ZIP_CODE_PATH = "PostalAddress/PostalCode";
		final String ADDRESSES_PATH = "PostalAddress/Street";
		
		// precondition
		if (addressNode == null)
			return;
		
		String bID = getNodeValue(addressNode, BUSINESS_ID_PATH);
		
		String contactName = getNodeValue(addressNode, CONTACT_NAME_PATH);
		
		
		ContactInfo ci = fJAXBWrapper.produceContactInfo(contactName, null, null, null);
		List cInfos = new ArrayList();
		if (ci != null)
			cInfos.add(ci);
		
		String city = getNodeValue(addressNode, CITY_PATH);
		String region = getNodeValue(addressNode, STATE_PATH);
		String zipCode = getNodeValue(addressNode, ZIP_CODE_PATH);

		Node countryNode = XPathAPI.selectSingleNode(addressNode, COUNTRY_PATH);
		String country = getNodeValue(countryNode, COUNTRY_CODE_PATH);
		
		List addresses = new ArrayList();
		NodeList addressesNodes = XPathAPI.selectNodeList(addressNode, ADDRESSES_PATH);
		for (int i = 0; i < addressesNodes.getLength(); i++)
		{
			Node streetNode = addressesNodes.item(i);
			addresses.add(streetNode.getFirstChild().getNodeValue());
		}
		
		Address address = fJAXBWrapper.produceAddress(country, 
														region, 
														city, 
														addresses, 
														zipCode);

		fJAXBWrapper.addCustomerRef(item,
				custRefType, 
				null,
				null,
				bID,
				cInfos,
				address);
	}
	
}
