package org.fmi.bdi.transformer.order;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Date;

import javax.xml.transform.TransformerException;


import org.apache.xpath.XPathAPI;
import org.fmi.bdi.commonstandard.orders.jaxb.IncomingMessageInfoType;
import org.fmi.bdi.db.DBHelper;
import org.fmi.bdi.document.GenericBDIDocumentWrapper;
import org.fmi.bdi.document.order.OrderBDIDocument;
import org.fmi.bdi.helpers.AppContext;
import org.fmi.bdi.helpers.BDIException;
import org.fmi.bdi.helpers.DBConnectionPool;
import org.fmi.bdi.helpers.Utils;
import org.fmi.bdi.transformer.ITransformer;
import org.w3c.dom.Document;
import org.w3c.dom.Node;


/**
 * @author IvoJekov
 *
 * handles common routines related to transformamition between incoming documents and the internal XML format
 */
public abstract class GenericXMLOrderTransformer implements ITransformer {

	protected GenericBDIDocumentWrapper fBDIDocWrapper = null;

	protected OrderBDIDocument fJAXBWrapper = null;

	/**
	 * Creates initial transformed document
	 *
	 */
	protected GenericXMLOrderTransformer() throws BDIException {
		fJAXBWrapper = new OrderBDIDocument();
	}

	/* (non-Javadoc)
	 * @see org.fmi.bdi.transformer.ITransformer#processDocument(Object)
	 */
	public void processDocument(Object doc) throws BDIException {
		fBDIDocWrapper = (GenericBDIDocumentWrapper) doc;

		AppContext app = AppContext.getInstance();
		DBConnectionPool pool = app.getDbConPool();

		Connection con = pool.getConnection();
		try {
			// first incoming document must be stored into the database
			int incomingMsgID = DBHelper.saveMessage(con, fBDIDocWrapper);
			fBDIDocWrapper.setIncomingMessageID(incomingMsgID);

			// transforms to internal format
			transformMessage();

			//finally the result must be stored too
			int transformedMsgID = DBHelper.saveMessage(con, fBDIDocWrapper);
			fBDIDocWrapper.setTransformedMessageID(transformedMsgID);

			con.commit();
		} catch (Exception e) {
			try {
				con.rollback();
			} catch (SQLException sqle) {
				throw new BDIException("Unexpected error occured: "
						+ sqle.getMessage(), sqle);
			}

			if (e instanceof BDIException)
				throw (BDIException) e;
			else
				throw new BDIException("Unexpected error occured: "
						+ e.getMessage(), e);

		} finally {
			pool.release(con);
		}
	}

	// common routins are done here
	protected void transformMessage() throws BDIException {
		storeIncomingMessageInfo();

		// transformation between incoming and internal format must occur
		performTransformation();

		// store the result of transformation into wrapper object
		storeTransformedResult();

	}

	protected void storeTransformedResult() throws BDIException {
		Document transformedDoc = fJAXBWrapper.getTransformedDocument();
		fBDIDocWrapper.setTransformedDocument(transformedDoc);
	}

	protected void storeIncomingMessageInfo() throws BDIException {
		int incomingMsgID = fBDIDocWrapper.getIncomingMessageID();
		String sourceSystem = fBDIDocWrapper.getSourceSystemNameVersion();

		IncomingMessageInfoType msgInfo = fJAXBWrapper.getRoot().getHeader()
				.getIncomingMessageInfo();
		msgInfo.setParentMsgID(incomingMsgID);
		msgInfo.setSourceSystem(sourceSystem);
		msgInfo.setReceivedDate(Utils.getDateTime(new Date()));
	}

	protected String getNodeValue(Node rootNode, String xPath)
			throws TransformerException {
		// precondition
		if (rootNode == null)
			return null;

		String value = null;
		Node node = XPathAPI.selectSingleNode(rootNode, xPath);
		if (node != null) {
			if (node.getNodeType() == Node.ATTRIBUTE_NODE)
				value = node.getNodeValue();
			else
				value = node.getFirstChild().getNodeValue();
		}

		return value;
	}

	protected abstract void performTransformation() throws BDIException;
}