/*
 * Created on 2004-10-6
 *
 */
package org.fmi.bdi.transformer.order;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBException;
import javax.xml.transform.TransformerException;

import org.apache.xpath.XPathAPI;
import org.fmi.bdi.commonstandard.orders.jaxb.Address;
import org.fmi.bdi.commonstandard.orders.jaxb.ContactInfo;
import org.fmi.bdi.commonstandard.orders.jaxb.DiscountTypeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.DocumentType;
import org.fmi.bdi.commonstandard.orders.jaxb.IncomingMessageInfoType;
import org.fmi.bdi.commonstandard.orders.jaxb.InstrTypeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.Item;
import org.fmi.bdi.commonstandard.orders.jaxb.PriceTypeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.ProductCodeConstants;
import org.fmi.bdi.commonstandard.orders.jaxb.TypeIDConstants;
import org.fmi.bdi.helpers.BDIException;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author IvoJekov
 *
 * Transforms incoming RosettaNet document into the internal XML format
 */
public class RosettaNetTransformer extends GenericXMLOrderTransformer
{
	public RosettaNetTransformer() throws BDIException
	{
		super();
	}
	
	/* (non-Javadoc)
	 * @see org.fmi.bdi.transformer.order.GenericOrderTransformer#performTransformation(org.fmi.bdi.document.GenericBDIDocumentWrapper)
	 */
	protected void performTransformation() throws BDIException
	{
		try
		{
			populateToFromInfo();
			adjustHeaderInfo();
			populateOrderHeaderInfo();
			populateOrderCurrencyInfo();
			populateDocumentRef();
			populateCustRef();
			populatePaymentInfo();
			populateCardInfo();
			populateDiscount();
			
			populateOrderItemsAndSummary();
			
			populateTaxInfo();
			populateTransportInfo();
		}
		catch (BDIException e)
		{
			throw e;
		}
		catch (Exception e)
		{
			throw new BDIException("Exception while transforming to internal format: " + e.getMessage(),
									e);
		}
	}
	
	private void populateOrderItemsAndSummary() throws TransformerException, 
														JAXBException,
														BDIException
	{
		final String ORDER_ITEMS_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/ProductLineItem";
		
		// CustRef
		final String CONTRACT_WITH_SUBPATH = "ContractInformation/primaryContractWith/PartnerDescription";
		final String PURCHASER_SUBPATH = "CustomerInformation/PartnerDescription";
		final String INSTALL_AT_SUBPATH = "installAt/PartnerDescription";
		final String SHIP_TO_SUBPATH = "shipTo/PartnerDescription";
		
		// DocRef
		final String DOC_REF_SUBPATH = "DocumentReference";

		// Tax
		final String TAX_SUBPATH = "TaxExemptStatus/TaxExemption";

		// Transport
		final String TRANSPORT_SUBPATH = "requestedEvent/TransportationEvent";
		
		// Quantity
		final String QUANTITY_SUBPATH = "OrderQuantity/requestedQuantity/ProductQuantity";
		final String UOM_SUBPATH = "GlobalProductUnitOfMeasureCode";

		// Quantity
		final String LEGACY_IN_SUBPATH = "LineNumber";
		
		// Price
		final String PRICE_AMOUNT_SUBPATH = "requestedUnitPrice/FinancialAmount/MonetaryAmount";
		final String PRICE_DESC_SUBPATH = "requestedUnitPrice/FinancialAmount/InvoiceChargeTypeCode";

		// Currency
		final String PRIMARY_CURRENCY_PATH = "requestedUnitPrice/FinancialAmount/GlobalCurrencyCode";

		// ProductInfo
		final String PRODUCT_SUBPATH = "ProductIdentification/PartnerProductIdentification";
		final String PRODUCT_NUMBER_SUBPATH = "ProprietaryProductIdentifier";
		final String PRODUCT_CODE_SUBPATH = "GlobalPartnerClassificationCode";
		
		float totalPrice = 0;
		int orderItemsCnt = 0;
		
		NodeList orderItems = XPathAPI.selectNodeList(fBDIDocWrapper.getIncomingDocument(),
														ORDER_ITEMS_PATH);
		orderItemsCnt = orderItems.getLength();
		for (int i = 0; i < orderItemsCnt; i++)
		{
			Node curItem = orderItems.item(i);
			Item newOI = fJAXBWrapper.produceOrderItem(i + 1);
			
			// Customer References are populated here
			// CustRef CBN
			Node contractInfo = XPathAPI.selectSingleNode(curItem, CONTRACT_WITH_SUBPATH);
			processPartnerDescription(contractInfo, TypeIDConstants.CBN, newOI);
			
			// CustRef PUR
			Node purchaser = XPathAPI.selectSingleNode(curItem, PURCHASER_SUBPATH);
			processPartnerDescription(purchaser, TypeIDConstants.PUR, newOI);
			
			// CustRef INS
			Node installedAtNode = XPathAPI.selectSingleNode(curItem, INSTALL_AT_SUBPATH);
			processPartnerDescription(installedAtNode, TypeIDConstants.INS, newOI);
			
			// CustRef SHP
			Node shipToNode = XPathAPI.selectSingleNode(curItem, SHIP_TO_SUBPATH);
			processPartnerDescription(shipToNode, TypeIDConstants.SHP, newOI);
			// end of cust refs
			
			// DocRefs here
			NodeList docRefs = XPathAPI.selectNodeList(curItem, DOC_REF_SUBPATH);
			processDocumentReferences(docRefs, newOI);
			
			// Tax here
			Node taxNode = XPathAPI.selectSingleNode(curItem, TAX_SUBPATH);
			processTax(taxNode, newOI);
			
			// Transport here
			Node transpEvent = XPathAPI.selectSingleNode(curItem, TRANSPORT_SUBPATH);
			processTransport(transpEvent, newOI);
			
			// ProductInfo
			// can not be null
			NodeList prodInfos = XPathAPI.selectNodeList(curItem, PRODUCT_SUBPATH);
			for (int j = 0; j < prodInfos.getLength(); j++)
			{
				Node curProdInfo = prodInfos.item(j);
				String prodCommodityCode = getNodeValue(curProdInfo, PRODUCT_CODE_SUBPATH);
				String prodNumber = getNodeValue(curProdInfo, PRODUCT_NUMBER_SUBPATH);
				
				fJAXBWrapper.addItemProductInfo(newOI, 
												ProductCodeConstants.BPN, 
												prodNumber, 
												prodCommodityCode, 
												null, 
												null);
			}
			
			// Price
			String price = getNodeValue(curItem, PRICE_AMOUNT_SUBPATH);
			float curPrice = 0;
			try
			{
				curPrice = Float.parseFloat(price);
			}
			catch (NumberFormatException nfe)
			{
				// nothing to due
				throw new BDIException("Invalid price supplied " 
						+ nfe.getMessage() + 
						" at item: " + newOI.getLineNumber(), 
						nfe);
			}
			String priceDesc = getNodeValue(curItem, PRICE_DESC_SUBPATH);
			fJAXBWrapper.addItemPrice(newOI, curPrice, priceDesc, PriceTypeConstants.BDP);;
			
			// Quantity
			String quantity = getNodeValue(curItem, QUANTITY_SUBPATH);
			int iQuantity = 0;
			try
			{
				iQuantity = Integer.parseInt(quantity);
			}
			catch (NumberFormatException nfe)
			{
				// nothing to due
				throw new BDIException("Invalid quantity supplied " 
						+ nfe.getMessage() + 
						" at item: " + newOI.getLineNumber(), 
						nfe);
			}
			String uom = getNodeValue(curItem, UOM_SUBPATH);
			fJAXBWrapper.setItemQuantity(newOI, iQuantity, uom);
			
			// calculate total order price
			totalPrice += iQuantity*curPrice;
			
			// Currency
			String currency = getNodeValue(curItem, PRIMARY_CURRENCY_PATH);
			if (currency != null)
				fJAXBWrapper.setCurrency(newOI, currency, null, null);
			
			// Legacy Item Number
			newOI.setLegacyItemNumber(getNodeValue(curItem, LEGACY_IN_SUBPATH));
		}
		
		fJAXBWrapper.setOrderSummary(totalPrice, null, orderItemsCnt);
	}
	
	private void populateDocumentRef() throws TransformerException, JAXBException
	{
		final String DOC_REF_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/DocumentReference";
		
		// could be null
		NodeList docRefNodes = XPathAPI.selectNodeList(fBDIDocWrapper.getIncomingDocument(), 
														DOC_REF_PATH);
		processDocumentReferences(docRefNodes, null);
	}
	
	private void populateCustRef() throws TransformerException, JAXBException
	{
		final String BILL_TO_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/AccountDescription/billTo/PartnerDescription";
		final String FINANCED_BY_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/AccountDescription/financedBy/PartnerDescription";
		final String INSTALLED_AT_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/installAt/PartnerDescription";
		final String SECONDARY_BUYER_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/SecondaryBuyer/PartnerDescription";
		final String SHIP_TO_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/shipTo/PartnerDescription";
		
		// 	bill to
		Node billNode = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), 
												 BILL_TO_PATH);
		processPartnerDescription(billNode, TypeIDConstants.BIL, null);
		
		// financed by
		Node financedByNode = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), 
													FINANCED_BY_PATH);
		processPartnerDescription(financedByNode, TypeIDConstants.FBY, null);
		
		// installed at
		Node installedAtNode = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), 
									INSTALLED_AT_PATH);
		processPartnerDescription(installedAtNode, TypeIDConstants.INS, null);
		
		// secondary buyer
		Node secondaryBuyerNode = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), 
									SECONDARY_BUYER_PATH);
		processPartnerDescription(secondaryBuyerNode, TypeIDConstants.PUR, null);
		
		// ship to
		Node shipToNode = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), 
				SHIP_TO_PATH);
		processPartnerDescription(shipToNode, TypeIDConstants.SHP, null);
	}
	
	private void populatePaymentInfo() throws TransformerException, JAXBException
	{
		final String FINANCIAL_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/FinancingTerms";
		final String TERM_SUBPATH = "GlobalFinanceTermsCode";
		final String PAYMENT_SUBPATH = "PaymentTerms";
		final String CODE_SUBPATH = "GlobalPaymentConditionCode";
		final String DAY_SUBPATH = "netTermsDay/DayOfMonth";
		final String PERIOD_SUBPATH = "netTermsDays/CountableAmount";
		
		// could be null
		NodeList financials = XPathAPI.selectNodeList(fBDIDocWrapper.getIncomingDocument(), 
				FINANCIAL_PATH);
		if (financials != null)
		{	
			for (int i = 0; i < financials.getLength(); i++)
			{
				Node curFin = financials.item(i);
				String paymentType = getNodeValue(curFin, TERM_SUBPATH);
				
				// could be null
				NodeList payments = XPathAPI.selectNodeList(curFin, PAYMENT_SUBPATH);
				if (payments != null)
				{
					for (int j = 0; j < payments.getLength(); j++)
					{
						Node curPayment = payments.item(i);
						String paymentCode = getNodeValue(curPayment, CODE_SUBPATH);
						String term1 = getNodeValue(curPayment, DAY_SUBPATH);
						String term2 = getNodeValue(curPayment, PERIOD_SUBPATH);
						
						fJAXBWrapper.addPayment(paymentType, paymentCode, term1, term2);
					}
				}
			}
		}
	}
	
	private void populateCardInfo() throws TransformerException, JAXBException
	{
		final String CREDIT_CARD_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/AccountDescription/CreditCard";
		final String CARD_HOLDER_SUBPATH = "cardHolderName/FreeFormText";
		final String CARD_NUM_SUBPATH = "creditCardIdentifier/ProprietaryReferenceIdentifier";
		final String CARD_TYPE_SUBPATH = "GlobalCreditCardClassificationCode";
		final String EXPIRE_SUBPATH = "expiryDate/DateStamp";	
		
		// could be null
		Node creditCard = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), 
				CREDIT_CARD_PATH);
		if (creditCard != null)
		{
			String cardType = getNodeValue(creditCard, CARD_TYPE_SUBPATH);
			String cardNum = getNodeValue(creditCard, CARD_NUM_SUBPATH);
			String expDate = getNodeValue(creditCard, EXPIRE_SUBPATH);
			String holderName = getNodeValue(creditCard, CARD_HOLDER_SUBPATH);
			
			fJAXBWrapper.setCardInfo(cardType, cardNum, holderName, expDate);			
		}
	}
	
	private void populateDiscount() throws TransformerException, JAXBException, BDIException
	{
		final String DISCOUNTS_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/FinancingTerms/PaymentTerms/Discounts";
		final String PERCENT_SUBPATH = "discountPercent/PercentAmount";
		final String DAY_SUBPATH = "discountDay/DayOfMonth";
		final String PERIOD_SUBPATH = "discountDays/CountableAmount";
		
		// could be null
		NodeList discounts = XPathAPI.selectNodeList(fBDIDocWrapper.getIncomingDocument(), 
				DISCOUNTS_PATH);
		if (discounts != null)
		{
			for (int i = 0; i < discounts.getLength(); i ++)
			{
				Node curDisc = discounts.item(i);
				String percentage = getNodeValue(curDisc, PERCENT_SUBPATH);
				String day = getNodeValue(curDisc, PERCENT_SUBPATH);
				String duration = getNodeValue(curDisc, PERCENT_SUBPATH);
				
				fJAXBWrapper.addDiscount(DiscountTypeConstants.REQ,
										null,
										null,
										null,
										percentage,
										day,
										duration);
				
			}
		}
	}
		
	/*
	 * For RosettaNet tax info is populated as instructions
	 */
	private void populateTaxInfo() throws TransformerException, JAXBException
	{
		final String TAX_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/TaxExemptStatus/TaxExemption";

		// could be null
		Node tax = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), 
				TAX_PATH);
		processTax(tax, null);
	}
	
	private void populateTransportInfo() throws TransformerException, JAXBException
	{
		final String TRANSPORT_INFO_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/requestedEvent/TransportationEvent";
		
		// could be null
		Node transport = XPathAPI.selectSingleNode(fBDIDocWrapper.getIncomingDocument(), 
				TRANSPORT_INFO_PATH);
		processTransport(transport, null);
	}
	
	private void adjustHeaderInfo() throws TransformerException
	{
		final String DOC_TYPE_PATH = "/Pip3A4PurchaseOrderRequest/GlobalDocumentFunctionCode";
		final String DOC_ID_PATH = "/Pip3A4PurchaseOrderRequest/thisDocumentIdentifier/ProprietaryDocumentIdentifier";
		final String SEND_DATE_PATH = "/Pip3A4PurchaseOrderRequest/thisDocumentGenerationDateTime/DateTimeStamp";
		
		DocumentType doc = fJAXBWrapper.getRoot().getHeader().getDocument();
		IncomingMessageInfoType msgInfo = fJAXBWrapper.getRoot().getHeader().getIncomingMessageInfo();
		
		// could be null
		String docType = getNodeValue(fBDIDocWrapper.getIncomingDocument(),	DOC_TYPE_PATH);
		if (docType != null)
			doc.setDocType(docType);
		// can not be null
		doc.setProprietaryDocID(getNodeValue(fBDIDocWrapper.getIncomingDocument(), DOC_ID_PATH));
		msgInfo.setSentDate(getNodeValue(fBDIDocWrapper.getIncomingDocument(), SEND_DATE_PATH));
	}
	
	private void populateOrderHeaderInfo() throws TransformerException, JAXBException
	{
		final String ORDER_TYPE_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/GlobalPurchaseOrderTypeCode";
		
		// could be null
		NodeList ohNodes = XPathAPI.selectNodeList(fBDIDocWrapper.getIncomingDocument(), 
				ORDER_TYPE_PATH);
		if (ohNodes != null)
		{
			for (int i = 0; i < ohNodes.getLength(); i++)
			{
				Node curOH = ohNodes.item(i);
				String orderType = curOH.getFirstChild().getNodeValue();
				
				fJAXBWrapper.addOrderHeader(orderType, null, null);
			}
		}
	}
	
	private void populateOrderCurrencyInfo() throws TransformerException, 
													JAXBException,
													BDIException
	{
		final String PRIMARY_CURRENCY_PATH = "/Pip3A4PurchaseOrderRequest/PurchaseOrder/totalAmount/FinancialAmount/GlobalCurrencyCode";

		// could be null
		String currency = getNodeValue(fBDIDocWrapper.getIncomingDocument(), PRIMARY_CURRENCY_PATH);
		if (currency != null)
		{
			fJAXBWrapper.setCurrency(null, currency, null, null);
		}
	}
	
	private void populateToFromInfo() throws TransformerException, JAXBException
	{
		final String FROM_ORG_PATH = "/Pip3A4PurchaseOrderRequest/fromRole/PartnerRoleDescription/PartnerDescription/BusinessDescription";
		final String TO_ORG_PATH = "/Pip3A4PurchaseOrderRequest/toRole/PartnerRoleDescription/PartnerDescription/BusinessDescription";		
		final String GLOBAL_SYSTEM_PATH = "/GlobalBusinessIdentifier";
		final String NAME_PATH = "/GlobalSupplyChainCode";

		// To parnter System
		String toGSID = getNodeValue(fBDIDocWrapper.getIncomingDocument(), TO_ORG_PATH + GLOBAL_SYSTEM_PATH);
		String toName = getNodeValue(fBDIDocWrapper.getIncomingDocument(), TO_ORG_PATH + NAME_PATH);
		fJAXBWrapper.setToPartner(toGSID, toName, null);
		
		// From Partner System
		String fromGSID = getNodeValue(fBDIDocWrapper.getIncomingDocument(), FROM_ORG_PATH + GLOBAL_SYSTEM_PATH);
		String fromName = getNodeValue(fBDIDocWrapper.getIncomingDocument(), FROM_ORG_PATH + NAME_PATH);
		fJAXBWrapper.setFromPartner(fromGSID, fromName, null);
	}
	
	
	//////////////////////////////////////////////////////////////////
	//
	// common helpers here
	//
	/////////////////////////////////////////////////////////////////
	private void processPartnerDescription(Node partnerDescr, 
										TypeIDConstants custRefType,
										Item item) 
								throws TransformerException, JAXBException
	{
		// business description
		final String BUSINESS_ID_PATH = "BusinessDescription/GlobalBusinessIdentifier";
		final String BUSINESS_NAME_PATH = "BusinessDescription/businessName/FreeFormText";
		final String PROPRIETARY_ID_PATH = "BusinessDescription/PartnerBusinessIdentification/ProprietaryBusinessIdentifier";
		final String PARNER_CODE_PATH = "GlobalPartnerClassificationCode";
		
		// contact info
		final String CONTACT_NAME_PATH = "ContactInformation/contactName/FreeFormText";
		final String EMAIL_PATH = "ContactInformation/EmailAddress";
		final String TELEPHONE_PATH = "ContactInformation/telephoneNumber/CommunicationsNumber";
		final String FAX_PATH = "ContactInformation/facsimileNumber/CommunicationsNumber";

		// address
		final String CITY_PATH = "PhysicalLocation/PhysicalAddress/cityName/FreeFormText";
		final String COUNTRY_PATH = "PhysicalLocation/PhysicalAddress/GlobalCountryCode";
		final String REGION_PATH = "PhysicalLocation/PhysicalAddress/regionName/FreeFormText";
		final String ZIP_CODE_PATH = "PhysicalLocation/PhysicalAddress/postOfficeBoxIdentifier/FreeFormText";
		final String ADDRESS_1_CODE_PATH = "PhysicalLocation/PhysicalAddress/addressLine1/FreeFormText";
		final String ADDRESS_2_CODE_PATH = "PhysicalLocation/PhysicalAddress/addressLine2/FreeFormText";
		final String ADDRESS_3_CODE_PATH = "PhysicalLocation/PhysicalAddress/addressLine3/FreeFormText";
		
		// precondition
		if (partnerDescr == null)
			return;
		
		String bID = getNodeValue(partnerDescr, BUSINESS_ID_PATH);
		String bName = getNodeValue(partnerDescr, BUSINESS_NAME_PATH);
		
		// "GlobalPartnerClassificationCode" take precedence
		String partnerCode = getNodeValue(partnerDescr, PARNER_CODE_PATH);
		if (partnerCode == null)
			partnerCode = getNodeValue(partnerDescr, PROPRIETARY_ID_PATH);
		
		String contactName = getNodeValue(partnerDescr, CONTACT_NAME_PATH);
		String email = getNodeValue(partnerDescr, EMAIL_PATH);
		String tel = getNodeValue(partnerDescr, TELEPHONE_PATH);
		String fax = getNodeValue(partnerDescr, FAX_PATH);
		
		ContactInfo ci = fJAXBWrapper.produceContactInfo(contactName, email, tel, fax);
		List cInfos = new ArrayList();
		if (ci != null)
			cInfos.add(ci);
		
		String city = getNodeValue(partnerDescr, CITY_PATH);
		String country = getNodeValue(partnerDescr, COUNTRY_PATH);
		String region = getNodeValue(partnerDescr, REGION_PATH);
		String zipCode = getNodeValue(partnerDescr, ZIP_CODE_PATH);
		
		List addresses = new ArrayList();
		String add1 = getNodeValue(partnerDescr, ADDRESS_1_CODE_PATH);
		if (add1 != null)
			addresses.add(add1);
		String add2 = getNodeValue(partnerDescr, ADDRESS_2_CODE_PATH);
		if (add2 != null)
			addresses.add(add2);
		String add3 = getNodeValue(partnerDescr, ADDRESS_3_CODE_PATH);
		if (add3 != null)
			addresses.add(add3);
		
		Address address = fJAXBWrapper.produceAddress(country, region, city, addresses, zipCode);

		fJAXBWrapper.addCustomerRef(item,
				custRefType, 
				bName,
				partnerCode,
				bID,
				cInfos,
				address);
	}
	
	private void processDocumentReferences(NodeList docRefNodes, 
										Item item) 
								throws TransformerException, JAXBException
	{
		final String DOC_ID_SUBPATH = "ProprietaryDocumentIdentifier";
		final String DOC_CODE_SUBPATH = "GlobalPartnerRoleClassificationCode";
		final String DOC_TYPE_SUBPATH = "GlobalDocumentReferenceTypeCode";
		
		// precondition
		if (docRefNodes == null)
			return;
		for (int i = 0; i < docRefNodes.getLength(); i++)
		{
			Node curDocRef = docRefNodes.item(i);
			
			String id = getNodeValue(curDocRef, DOC_ID_SUBPATH);
			String type = getNodeValue(curDocRef, DOC_TYPE_SUBPATH);
			String code = getNodeValue(curDocRef, DOC_CODE_SUBPATH);
			
			fJAXBWrapper.addDocumentRef(item, id, type, code, null);
		}
	}
	
	/*
	 * For RosettaNet tax info is populated as instructions
	 */
	private void processTax(Node tax, Item item) throws TransformerException, JAXBException
	{
		final String TAX_CODE_SUBPATH = "GlobalTaxExemptionCode";
		final String TAX_CERT_ID_SUBPATH = "taxExemptionCertificationIdentifier/ProprietaryReferenceIdentifier";
		
		// could be null
		if (tax != null)
		{
			Node nodeCode = XPathAPI.selectSingleNode(tax, TAX_CODE_SUBPATH);
			String code = nodeCode.getNodeName();
			String value = nodeCode.getFirstChild().getNodeValue();
			
			fJAXBWrapper.addInstruction(item,
										InstrTypeConstants.MARKER,
										"TAX",
										code,
										value,
										null);
			
			Node nodeCertId = XPathAPI.selectSingleNode(tax, TAX_CERT_ID_SUBPATH);
			code = nodeCertId.getParentNode().getNodeName();
			value = nodeCertId.getFirstChild().getNodeValue();
			
			fJAXBWrapper.addInstruction(item,
										InstrTypeConstants.MARKER,
										"TAX",
										code,
										value,
										null);			
		}
	}
	
	private void processTransport(Node transportationEvent, Item item) throws TransformerException, JAXBException
	{
		final String TRANSPORT_DATE_SUBPATH = "DateStamp";
		final String TRANSPORT_MODE_SUBPATH = "GlobalTransportEventCode";
		
		if (transportationEvent != null)
		{
			String date = getNodeValue(transportationEvent, TRANSPORT_DATE_SUBPATH);
			String mode = getNodeValue(transportationEvent, TRANSPORT_MODE_SUBPATH);
			
			fJAXBWrapper.setTransport(item, date, mode);			
		}
	}
	
}
