<?xml version="1.0" encoding="UTF-8"?>
<!--
=====================================================================
                            jaxb.xsl
This XSLT stylesheet
    - converts namespace-prefix to 'xs:' 
    - applies JAXB typesafe enumeration to GS schemas.
=====================================================================
-->
<xsl:stylesheet
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:jaxb="http://java.sun.com/xml/ns/jaxb" 
	xmlns:xjc="http://java.sun.com/xml/ns/jaxb/xjc" 
	exclude-result-prefixes="xjc"
	version="1.0">
	
	<!-- set the output method -->
	<xsl:output method="xml"
		version="1.0"
		indent="yes"/>

	<xsl:strip-space elements="*"/>
	
	<!-- match the root of the document -->
	<xsl:template match="/">
		<xsl:apply-templates/>
	</xsl:template>
	
	<!-- match the schema element -->
	<xsl:template match="xs:schema">
	    <xsl:copy>
	        <xsl:apply-templates select="@*"/>
	        <!-- needed for JAXB inline annotation -->
	        <xsl:attribute name="jaxb:version">1.0</xsl:attribute>
    		<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>
	
	<!-- match the simpleType element -->
	<xsl:template match="xs:simpleType">
	    <xsl:copy>
	        <xsl:apply-templates select="@*"/>
	        <xsl:choose>
                <xsl:when test="descendant::xs:enumeration and xs:annotation">
                    <!-- enumerations and an annotation exist -->     
                    <xsl:apply-templates select="xs:annotation" mode="applyEnumeration"/>
                    <xsl:apply-templates select="*[name() != 'xs:annotation']"/>
                </xsl:when>  
                <xsl:when test="descendant::xs:enumeration">
                    <!-- enumerations exist and an annotation does not exist -->
                    <xs:annotation>
                        <xs:appinfo>
                            <jaxb:typesafeEnumClass name="{concat(ancestor::xs:attribute/@name, 'Constants')}">
                                <xsl:for-each select="descendant::xs:enumeration">
                                    <jaxb:typesafeEnumMember 
                                        name="{@value}" value="{@value}"/>
                                </xsl:for-each>
                            </jaxb:typesafeEnumClass>    
                        </xs:appinfo>
                    </xs:annotation>
                    <xsl:apply-templates/>
                </xsl:when>  
                <xsl:otherwise>
                    <xsl:apply-templates/>
                </xsl:otherwise>
            </xsl:choose>
		</xsl:copy>
	</xsl:template>
	
	<!-- match the annotation element -->
	<xsl:template match="xs:annotation" mode="applyEnumeration">
	    <xsl:copy>
	        <xsl:apply-templates/>
	        <!-- insert appinfo -->
            <xs:appinfo>
               <jaxb:typesafeEnumClass name="{concat(ancestor::xs:attribute/@name, 'Constants')}">
                   <xsl:for-each select="parent::*/descendant::xs:enumeration">
                       <jaxb:typesafeEnumMember 
                           name="{@value}" value="{@value}"/>
                        </xsl:for-each>
                </jaxb:typesafeEnumClass>    
             </xs:appinfo>
        </xsl:copy>     
	</xsl:template>
	
    <!-- match every attribute and element node 
        Note: does a deep copy, allows templates above to become instanciated -->
	<xsl:template match="@*|*">
		<xsl:copy>
			<xsl:apply-templates select="@*"/>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>
        
</xsl:stylesheet>
