/*
 * Created on 2004-8-1
 *
 */
package org.fmi.bdi.tests;

import junit.framework.Test;
import junit.framework.TestSuite;

import java.io.BufferedReader;
import java.io.FileReader;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import org.fmi.bdi.helpers.DBConnectionPool;

/**
 * @author IvoJekov
 *
*/
public class TestJDBC extends BaseTest
{

	private DBConnectionPool conPool = null;
	
	public void testLargeObjectInsert() throws Exception 
	{
		Connection con = conPool.getConnection();
		
		StringBuffer fileContent = new StringBuffer();
		BufferedReader br = new BufferedReader(new FileReader("F:/Personal Info/Dimplomna rabota/Source/bdi/testcases/RosettaNet/3A4PurchaseOrderRequestMessageBase_XSD.xml"));
		String line = null;
		while ((line = br.readLine()) != null)
		{
			fileContent.append(line + "\n");
		}
		
		int msgId = 1000;
		PreparedStatement ps = null;
		try
		{
			ps = con.prepareStatement("delete from \"T_MESSAGE\" where \"MESSAGE_ID\" = ?");
			ps.setInt(1, msgId);
			ps.executeUpdate();
			
			ps = con.prepareStatement("insert into \"T_MESSAGE\" (\"MESSAGE_ID\", \"DOCUMENT_ID\", \"MESSAGE_TYPE_ID\", \"MESSAGE_VALUE\") values(?, 1, 1, ?)");
			ps.setInt(1, msgId);
			ps.setString(2,fileContent.toString());
			ps.executeUpdate();
			
			con.commit();
			
			System.out.println("\nInsert large string test passed.\n");
		}
		catch (SQLException e)
		{
			con.rollback();
			throw e;
		}
		finally
		{
			try
			{
				if (ps != null)
					ps.close();
			}
			finally
			{
				conPool.release(con);
			}
		}
		
	}
	
	public void testSP() throws Exception 
	{
		Connection con = conPool.getConnection();
		
		StringBuffer fileContent = new StringBuffer();
		BufferedReader br = new BufferedReader(new FileReader("F:/Personal Info/Dimplomna rabota/Source/bdi/testcases/RosettaNet/3A4PurchaseOrderRequestMessageBase_XSD.xml"));
		String line = null;
		while ((line = br.readLine()) != null)
		{
			fileContent.append(line + "\n");
		}
		
		CallableStatement ps = null;
		try
		{
			ps = con.prepareCall("{?= call store_message(?, ?, ?, ?, ?)}");
			ps.registerOutParameter(1, Types.INTEGER);
			ps.setInt(2,1);
			ps.setInt(3,1);
			ps.setString(4,fileContent.toString());
			
//			ps.setNull(5, Types.INTEGER);
//			ps.setNull(6, Types.INTEGER);
			ps.setInt(5, 1);
			ps.setInt(6, 1);
			
			
			ps.execute();
			
			System.out.println("Latest MESSAGE_ID:" +
					"\t" + ps.getInt(1));
			
			con.commit();
			
			System.out.println("\nDB Function invocation test passed.\n");
		}
		catch (SQLException e)
		{
			con.rollback();
			throw e;
		}
		finally
		{
			try
			{
				if (ps != null)
					ps.close();
			}
			finally
			{
				conPool.release(con);
			}
		}
		
	}
	
	public void testSelectQuery() throws Exception 
	{
		retrieveDomainTableContent("TD_VALIDATION_TYPE");
		retrieveDomainTableContent("TD_DOCUMENT_TYPE");
		retrieveDomainTableContent("TD_MESSAGE_TYPE");
		retrieveDomainTableContent("TD_RELATION_TYPE");
		
		System.out.println("\nSelect from database test passed.\n\n\n");
	}

	
	private void retrieveDomainTableContent(String tableName) throws Exception 
	{
		String pkColumnName = tableName.substring(3) + "_ID";
		
		Connection con = conPool.getConnection();
		
		PreparedStatement ps = null;
		try
		{
			ps = con.prepareStatement("select * from \"" + tableName + "\"");
			ResultSet rs = ps.executeQuery();
			
			System.out.println("\nTable " + tableName + " content:");
			while (rs.next())
			{
				System.out.println(rs.getString(pkColumnName) +
									"\t" + rs.getString("CODE") +
									"\t" + rs.getString("DESCR"));
			}
		}
		finally
		{
			try
			{
				if (ps != null)
					ps.close();
			}
			finally
			{
				conPool.release(con);
			}
		}
	}
	
	
	/**
	 * Setup for the testcase
	 */
	protected void setUp() throws Exception
	{
		super.setUp();
		conPool = fContext.getDbConPool();
	}
	/**
	 *  
	 */
	protected static Test suite() {
		return new TestSuite(TestJDBC.class);
	}
	
}
