/*
 * Created on 2004-8-5
 *
 */
package org.fmi.bdi.tests;

import java.io.File;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.fmi.bdi.db.domains.IDocumentType;
import org.fmi.bdi.document.GenericBDIDocumentWrapper;
import org.fmi.bdi.helpers.BDIException;
import org.fmi.bdi.helpers.Utils;
import org.fmi.bdi.transformer.ITransformer;

/**
 * @author IvoJekov
 *
*/
public class TestTransform extends BaseTest
{
	public void testRosettaNetCorrect()
	{
		// valid
		transformPOMessage("C:/Diploma/Source/bdi/src/examples/RosettaNet/3A4PurchaseOrderRequest_2LineItems.xml", 1, true);
	}
	
	public void testRosettaNetIncorrect()
	{
		// invalid
		transformPOMessage("C:/Diploma/Source/bdi/src/examples/RosettaNet/3A4PurchaseOrderRequest_Invalid.xml", 1, false);
	}
	
	public void testCXMLCorrect() throws Exception
	{
		transformPOMessage("C:/Diploma/Source/bdi/src/examples/cXML/OrderRequest_Normal.xml", 2, true);
	}
	
	public void testCXMLWarn() throws Exception
	{
		// second, warning
		transformPOMessage("C:/Diploma/Source/bdi/src/examples/cXML/OrderRequest_Price_Do_Not_Match.xml", 2, true);
	}
	
	public void testCXMLInorrect() throws Exception
	{
		transformPOMessage("C:/Diploma/Source/bdi/src/examples/cXML/OrderRequest_Invalid.xml", 2, false);
	}
	
	
	// common helper
	private void transformPOMessage(String fileName, int ssId, boolean shouldSuceed)
	{
		Exception exc = null;
		try
		{
			// test parsing
			File xml = new File(fileName);
			GenericBDIDocumentWrapper orderWrapper = new GenericBDIDocumentWrapper(ssId, IDocumentType.PURCHASE_ORDER, xml);
			
			// test transforming
			String transformerClassName = orderWrapper.getTransformerJavaClass();
			ITransformer trans = (ITransformer) Class.forName(transformerClassName).newInstance();
			trans.processDocument(orderWrapper);
			
			String transformedMsg = Utils.xmlToString(orderWrapper.getTransformedDocument());
			System.out.println(transformedMsg);
		}
		catch (Exception e)
		{
			exc = e;
			if (e instanceof BDIException)
			{
				if ( ((BDIException) e).getInnerException() != null )
					exc = ((BDIException) e).getInnerException();
			}
			
		}
		
		if ((shouldSuceed && exc != null)
			|| (!shouldSuceed && exc == null))
		{
			// test has failed 
			exc.printStackTrace();
			fail(exc.getMessage());
		}
	}
	
	/**
	 *  
	 */
	protected static Test suite() {
		return new TestSuite(TestTransform.class);
	}
	
}
