package org.fmi.bdi.web.multipart;


/**
 * @author Ivo Jekov
 *
 *
 */

import java.io.IOException;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;


/**
*  Wrapper for MultipartRequest
*/
public class ServletMultipartRequest extends MultipartRequest
{
    //~ Constructors ---------------------------------------------------------------------

    /**
     * @deprecated Replaced by ServletMultipartRequest(ServletRequest, String, int, String)
     */
    public ServletMultipartRequest(HttpServletRequest request, String strSaveDirectory)
        throws IllegalArgumentException, IOException
    {
        super(null, request.getContentType(), request.getContentLength(),
            request.getInputStream(), strSaveDirectory, MultipartRequest.MAX_READ_BYTES);
    }

    /**
     * @deprecated Replaced by ServletMultipartRequest(ServletRequest, String, int, String)
     */
    public ServletMultipartRequest(HttpServletRequest request, String strSaveDirectory,
        int intMaxReadBytes) throws IllegalArgumentException, IOException
    {
        super(null, request.getContentType(), request.getContentLength(),
            request.getInputStream(), strSaveDirectory, intMaxReadBytes);
    }

    /**
     * @deprecated Replaced by ServletMultipartRequest(ServletRequest, int, String)
     */
    public ServletMultipartRequest(HttpServletRequest request, int intMaxReadBytes)
        throws IllegalArgumentException, IOException
    {
        super(null, request.getContentType(), request.getContentLength(),
            request.getInputStream(), intMaxReadBytes);
    }

    /**
     * Standard Constructor
     *
     * @param request                                The ServletRequest will be used to initialise the MultipartRequest super class.
     * @param strSaveDirectory                The temporary directory to save the file from where they can then be moved to wherever by the
     *                                                                 calling process.  <b>If you specify <u>null</u> for this parameter, then any files uploaded
     *                                                                will be silently ignored.</B>
     * @param intMaxReadBytes                Overrides the MAX_BYTES_READ value, to allow arbitrarily long files.
     * @param encoding              Sets the encoding to use. If null, ISO-8859-1 will be used.
     *
     * @exception IllegalArgumentException         If the request.getContentType() does not contain a Content-Type of "multipart/form-data" or the boundary is not found.
     * @exception IOException                                If the request.getContentLength() is higher than MAX_READ_BYTES or strSaveDirectory is invalid or cannot be written to.
     *
     * @see MultipartRequest#MAX_READ_BYTES
     */
    public ServletMultipartRequest(ServletRequest request, String strSaveDirectory,
        int intMaxReadBytes, String encoding)
        throws IllegalArgumentException, IOException
    {
        super(null, //debug
            request.getContentType(), request.getContentLength(),
            request.getInputStream(), strSaveDirectory, intMaxReadBytes, encoding);
    }

    /**
     * Memory Constructor
     *
     * @param request                                The ServletRequest will be used to initialise the MultipartRequest super class.
     * @param intMaxReadBytes                Overrides the MA_BYTES_READ value, to allow arbitrarily long files.
     * @param encoding              Sets the encoding to use. If null, ISO-8859-1 will be used.
     *
     * @exception IllegalArgumentException         If the request.getContentType() does not contain a Content-Type of "multipart/form-data" or the boundary is not found.
     * @exception IOException                                If the request.getContentLength() is higher than MAX_READ_BYTES or strSaveDirectory is invalid or cannot be written to.
     *
     * @see MultipartRequest#MAX_READ_BYTES
     */
    public ServletMultipartRequest(ServletRequest request, int intMaxReadBytes,
        String encoding) throws IllegalArgumentException, IOException
    {
        super(null, request.getContentType(), request.getContentLength(),
            request.getInputStream(), intMaxReadBytes, encoding);
    }
}
